;----------------------------------------------------------------------------
; Tile Map Test routine by CrASH_Man
;----------------------------------------------------------------------------
   .include crash82.inc

himem           = $E000   ; relocate GRAPH_MEM and APD_BUF here
                          ; ($600 bytes page aligned)

graphmem        = $E000   ; new GRAPH_MEM

graphic_set     = $E300   ; within APD_BUF, page aligned
graphics        = $E400   ; within APD_BUF, page aligned
graphics_page   = $E4     ; page address of above

   .DB   "map tester by crashman", 0

   DI                 ; required in v3.0 - disable interrupts

;   LD    A, $05       ; required in v3.0 - sets down mode on LCD controller
;   OUT   ($10), A     ;

   XOR   A
   LD    (sox), A
   LD    (ssx), A
   LD    (soy), A
   LD    (ssy), A
   LD    (framenum), A


   LD    HL, map         ; map must be vertical too
   LD    DE, mapbuffer

   LD    C, $10
map_copyvloop:
   PUSH  DE
   LD    B, $10
map_copyhloop:
   LD    A, (HL)
   ADD   A, A
   ADD   A, A
   ADD   A, A
   LD    (DE), A
   INC   HL

   LD    A, 16

   ADD   A, E            ; add A to DE
   LD    E, A
   ADC   A, D
   SUB   E
   LD    D, A
   DJNZ  map_copyhloop
   POP   DE
   INC   DE
   DEC   C
   JR    NZ, map_copyvloop

   CALL  init

   LD    HL, graphic_source
   LD    DE, graphic_set
   LD    BC, 32*8
   LDIR

main_loop:
   LD    HL, framenum
   INC   (HL)
   RES   2, (HL)
   LD    A, (HL)
   OR    A
   JR    NZ, noscrollthis

   LD    HL, graphic_set+scrollthis-graphic_source    ; where in tileset?
   LD    B, 8
scrollthisloop:
   RLC   (HL)
   INC   HL
   DJNZ  scrollthisloop

noscrollthis:
   CALL  render_map
;   CALL  CR_GRBCopy   ; nope, not needed in v3.0 :)

   CALL  processkeys

   BIT   4, D
   JR    NZ, no_halt
   EI
   HALT
   DI
no_halt:

   LD    HL, soy
   LD    BC, ssy
   LD    E, 8
   BIT   3, D
   CALL  NZ, process_up
   BIT   0, D
   CALL  NZ, process_down

   LD    HL, sox
   LD    BC, ssx
   LD    E, 4
   BIT   1, D
   CALL  NZ, process_up
   BIT   2, D
   CALL  NZ, process_down

   BIT   7, D
   JR    NZ, exit

   JR    main_loop

process_up:
   DEC   (HL)
   RET   P
   LD    A, (BC)
   LD    (HL), A
   OR    A
   RET   Z
   DEC   A
   LD    (BC), A
   LD    (HL), 7
   RET
process_down:
   LD    A, (BC)
   CP    E
   RET   Z
   INC   (HL)
   BIT   3, (HL)
   RET   Z
   LD    (HL), 0
   LD    A, (BC)
   INC   A
   LD    (BC), A
   RET

exit:

;- init ---------------------------------------------------------------------

init:
   LD    HL, himem

   LD    DE, GRAPH_MEM
   LD    BC, 768
   CALL  reloc_f

   LD    DE, APD_BUF
   LD    BC, 768

;----------------------------------------------------------------------------
;[ reloc_f ]
;----------------------------------------------------------------------------
;input:
; DE->region1
; HL->region2
; BC=length of buffer
;returns:
; DE->region1+length
; HL->region2+length
reloc_f:
   LD    A, (DE)
   LDI
   DEC   HL
   LD    (HL), A
   INC   HL
   JP    PE, reloc_f
   RET

framenum:
   .DB   0

;----------------------------------------------------------------------------
;[ Processkeys ]                                   [ 39 bytes ] [ CrASH_Man ]
;----------------------------------------------------------------------------
; Reads the status of UP, DOWN, LEFT, RIGHT, 2nd, MODE, DEL, and ZOOM.
;
; The status goes into KSTAT_CUR and KSTAT_CHG.
;
; 76543210 -> 0=down, 1=left, 2=right, 3=up, 4=2nd, 5=mode, 6=del, 7=zoom
;
; If the key is held, bit in KSTAT_CUR is set.
; If the key was just pressed or released, bit in KSTAT_CHG is set.
;
; 186 clocks
;
; inputs:
;  none
; outputs:
;  D=current keys, A=changed keys, HL->key_chg, B=$FF, C=KEY_PORT

KEY_PORT = $01

KSTAT_CUR:
   .DB   0
KSTAT_CHG:
   .DB   0

processkeys:
   LD    BC, $FF00 + KEY_PORT          ; Reset Mask and port number
   OUT   (C), B                        ; Reset port

   LD    HL, %11111110 10111111        ; Direction mask and Functions mask
   OUT   (C), L                        ; Set port for Functions
   IN    A, (KEY_PORT)                 ; Get data

   LD    D, A                          ; Get state of zoom
   AND   %00000100                     ;
   ADD   A, B                          ;

   OUT   (C), B                        ; Reset port

   LD    A, D                          ; Set 7654xxxx to:
   RRA                                 ;  7=zoom, 6=del, 5=mode, 4=2nd
   OR    %00001111                     ;
   LD    D, A                          ;

   OUT   (C), H                        ; Set port for Direction
   IN    A, (KEY_PORT)                 ; Get data

   OR    %11110000                     ; Set xxxx3210 to directions
   XOR   D                             ; Stick all 8 key states in one byte
   LD    D, A                          ; Back up current keys

   LD    HL, KSTAT_CUR                 ; Get changed keys from previous state
   XOR   (HL)                          ;
   LD    (HL), D                       ; Save current keys
   INC   HL                            ;
   LD    (HL), A                       ; Save changed keys

   RET

waitkey:
   CALL  GET_KEY
   OR    A
   JR    Z, waitkey
   RET
;----------------------------------------------------------------------------
ssx:
   .DB   0
sox:
   .DB   0
ssy:
   .DB   0
soy:
   .DB   0

.include tilemap.inc
;----------------------------------------------------------------------------
mapbuffer:
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
.DB   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

map:
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6
.DB    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15
.DB    7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7

graphic_source:
.DB   %00000000 ; 0
.DB   %00000000
.DB   %00000000
.DB   %00000000
.DB   %00000000
.DB   %00000000
.DB   %00000000
.DB   %00000000

.DB   %11111111 ; 1
.DB   %11000001
.DB   %11111101
.DB   %11111101
.DB   %11111101
.DB   %11111101
.DB   %11111111
.DB   %11111111

.DB   %00010001 ; 2
.DB   %11111111
.DB   %01000100
.DB   %11111111
.DB   %00010001
.DB   %11111111
.DB   %01000100
.DB   %11111111

.DB   %10011001 ; 3
.DB   %11001100
.DB   %01100110
.DB   %00110011
.DB   %10011001
.DB   %11001100
.DB   %01100110
.DB   %00110011

.DB   %11001100 ; 4
.DB   %11001100
.DB   %00110011
.DB   %00110011
.DB   %11001100
.DB   %11001100
.DB   %00110011
.DB   %00110011

.DB   %10001000 ; 5
.DB   %00000000
.DB   %00100010
.DB   %00000000
.DB   %10001000
.DB   %00000000
.DB   %00100010
.DB   %00000000

.DB   %11110000 ; 6
.DB   %11010000
.DB   %11110000
.DB   %11110000
.DB   %00001111
.DB   %00001101
.DB   %00001111
.DB   %00001111

scrollthis:
.DB   %10000001 ; 7
.DB   %01000010
.DB   %00100100
.DB   %00011000
.DB   %00011000
.DB   %00100100
.DB   %01000010
.DB   %10000001

.DB   %11111111 ; 8
.DB   %10000001
.DB   %10001101
.DB   %10000101
.DB   %10000001
.DB   %10000001
.DB   %10000001
.DB   %11111111

.DB   %11111111 ; 9
.DB   %10000001
.DB   %11000011
.DB   %11100101
.DB   %11111011
.DB   %11110101
.DB   %11111011
.DB   %11111111

.DB   %00111100 ;10
.DB   %01110010
.DB   %11100001
.DB   %11100001
.DB   %11110011
.DB   %11111111
.DB   %01111110
.DB   %00111100

.DB   %10101010 ;11
.DB   %01010101
.DB   %10101010
.DB   %01010101
.DB   %10101010
.DB   %01010101
.DB   %10101010
.DB   %01010101

.DB   %01111101 ;12
.DB   %01110101
.DB   %01010101
.DB   %01001101
.DB   %01100101
.DB   %01010101
.DB   %01011101
.DB   %01111101

.DB   %01111101 ;13
.DB   %01110101
.DB   %01010101
.DB   %01001101
.DB   %11111111
.DB   %00000001
.DB   %00000001
.DB   %11111111

.DB   %11111111 ;14
.DB   %10000001
.DB   %10010001
.DB   %10011001
.DB   %10010001
.DB   %10000001
.DB   %11111111
.DB   %11111111

.DB   %11111111 ;15
.DB   %00100010
.DB   %00010001
.DB   %00100010
.DB   %00010001
.DB   %00100010
.DB   %11111111
.DB   %11111111
