;----------------------------------------------------------------------------
;[ PutSprClp ] [ABCDEFIX]                         [ 177 bytes ] [ CrASH_Man ]
;----------------------------------------------------------------------------
; Draws a sprite with an AND/XOR mask with clipping
;
; parameters: HL -> sprite, (B,C) = coordinates
; returns:    Puts sprite in page-aligned graphmem

ScreenWidthTile = 12
ScreenHeightTile = 8
DefaultSpriteHeight = 8

PutSprClp:
   XOR   A                             ; Clear A/CY
__Change_1 = $+1
   LD    DE, DefaultSpriteHeight       ; D=0, E=Height

   OR    C                             ; If Y<0
   JP    M, _VSR_NoBotClp              ; No bottom clip.

   LD    A, ScreenHeightTile*8-1       ; Is C is offscreen?
   SUB   C                             ;
   RET   C                             ;

__Change_2 = $+1
   CP    DefaultSpriteHeight-1         ; If Y+7<64
   JR    NC, _VSR_NoVertClp            ; No vertical clip.
   INC   A                             ;
   LD    E, A                          ;
   JR    _VSR_NoVertClp                ; Height=64-Y
 
_VSR_NoBotClp:                         ;
__Change_3 = $+1
   CP    -(DefaultSpriteHeight-1)      ; Is Y offscreen?
   RET   C                             ;

   ADD   A, E                          ; Find how many lines
   LD    C, A                          ; to actually draw
   SUB   E                             ;

   NEG                                 ;
   ADD   A, A                          ;
   LD    E, A                          ;
   ADD   HL, DE                        ; Move HL down
   LD    E, C                          ; by -Y lines
   LD    C, D                          ;

_VSR_NoVertClp:
   LD    A, $77                        ; OP code for
   LD    (_VSR_OPchg_1), A             ;   LD   (HL), A
   LD    (_VSR_OPchg_2), A             ;

   LD    A, B                          ; A=X
   EX    AF, AF'                       ; save copy of X
   LD    A, B                          ;
   LD    B, E                          ; B=Number of row to draw

   EX    DE, HL                        ; DE->Sprite

   OR    A                             ; Is X>0?
   JP    M, _VSR_NoRightClp            ;

   CP    ScreenWidthTile*8-7           ; Is X<89?
   JR    C, _VSR_ClpDone               ;
   CP    ScreenWidthTile*8             ;
   RET   NC                            ;

   LD    HL, _VSR_OPchg_1              ; Modify to NOP
   JR    _VSR_ClpModify                ;

_VSR_NoRightClp:
   CP    -7                            ; Is X offscreen?
   RET   C                             ;

   LD    HL, _VSR_OPchg_2              ; Modify to NOP
_VSR_ClpModify:
   LD    (HL), $00                     ;

_VSR_ClpDone:
   ADD   A, 8                          ; Change X to signed (-8 to 247)
   AND   %11111000                     ; Mask out the tile offset
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   LD    L, A                          ; HL=(X/8)*64+Y
   AND   %00000111                     ;
   LD    H, A                          ;
   XOR   L                             ;
   ADD   A, C                          ;
   LD    L, A                          ;

   LD    A, H                          ; HL->graphmem[(X/8)*64+Y+64]
   ADD   A, (graphmem>>8)              ;
   LD    H, A                          ;

   PUSH  HL                            ;
   EXX                                 ; setup shadows

   EX    AF, AF'                       ; A=X
   AND   %00000111                     ; Modify the number of times to
   LD    (_VSR_OPchg_4), A             ; rotate the bits
   XOR   %00000111                     ;
   LD    (_VSR_OPchg_3), A             ;

   LD    HL, MaskTable                 ; Get Mask Information
   ADD   A, L                          ; HL->Mask[(B&7)^7]
   LD    L, A                          ;
   ADC   A, H                          ;
   SUB   L                             ;
   LD    H, A                          ;

   LD    A, (HL)                       ; A=mask

   POP   HL                            ; HL'->graphmem[(X/8)*64+Y]
   LD    DE, -64                       ;
   ADD   HL, DE                        ;

   LD    D, A                          ; D'=%00001111
   CPL                                 ; E'=%11110000
   LD    E, A                          ;
   EXX                                 ;

;register mapping
;
;ACB'C'=tmp
;B=loop counter
;
;DE->sprite/sprite mask
;D'=mask %00001111
;E'=mask %11110000
;HL'->graphmem
;HL->graphmem+64

;-----------------------------------------------------------------------------
; 1 pass, 191 clockz/loop, 46 bytes, up to 4 dynamically modified bytes

_VSR_LineLoop:
   LD    A, (DE)                       ; xor mask
   INC   DE                            ; move pointer
_VSR_OPchg_3 = $+1
   JR    _VSR_OPchg_3                  ; as many as we need
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA

   EXX
   LD    C, A                          ; C' = rotated
   AND   D                             ; mask part we need
   LD    B, A                          ; B' = left side of sprite
   XOR   C                             ;
   EXX
   LD    C, A                          ; C = right side of sprite
   
   LD    A, (DE)                       ; and mask
   INC   DE                            ; move pointer
   EXX
_VSR_OPchg_4 = $+1
   JR    _VSR_OPchg_4                  ; as many as we need
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   
   RLCA
   LD    C, A                          ; C' = rotated
   OR    E                             ; A = masked
   AND   (HL)                          ; mask with screen
   XOR   B                             ; xor with screen
_VSR_OPchg_2:
   LD    (HL), A                       ; save to screen
   INC   L                             ; move screen pointer
   
   LD    A, C                          ; A = rotated
   OR    D                             ; A = masked
   EXX
   AND   (HL)                          ; mask with screen
   XOR   C                             ; xor with screen
_VSR_OPchg_1:
   LD    (HL), A                       ; save to screen
   INC   L                             ; move screen pointer

   DJNZ  _VSR_LineLoop                 ; loopz :P
   RET

MaskTable:
   .DB   %00000001
   .DB   %00000011
   .DB   %00000111
   .DB   %00001111
   .DB   %00011111
   .DB   %00111111
   .DB   %01111111
   .DB   %11111111

;#IFNDEF NO_MOD_AX
;;----------------------------------------------------------------------------
;;[ SetSpriteHeight ] [A]                           [ 16 bytes ] [ CrASH_Man ]
;;----------------------------------------------------------------------------
;; Changes the default sprite size
;;
;; parameters: A = New Sprite Height
;; returns:    Self modified routine.
;;
;SetSpriteHeight:LD   (__Change_1+1), A
;                LD   (__Change_4+2), A
;                DEC  A
;                LD   (__Change_2+1), A
;                NEG
;                LD   (__Change_3+1), A
;                RET
;#ENDIF
