;----------------------------------------------------------------------------
;[[ render_map v3.0 ]] (c) 2000 CrASHware                      [882] (56466)
;----------------------------------------------------------------------------
; Copies current graphmem to LCD and draws a full screen of a 8x8 tile map
;                                             starting at (ssx:sox, ssy:soy)
;
; special thanks to Dave Phillips and Kirk Meyer for optimization ideas
;
; Interrupts must be disabled
; sox = screen x offset pixel offset (0-7)
; soy = screen y offset pixel offset (0-7)
; ssx = screen tile x
; ssy = screen tile y
;
; mapbuffer      = array of map data, with data being multiplied by 8
; graphic_set   -> array of NUM_TILES tiles, 8 bytes each
; graphics      -> free buffer of [256 + NUM_TILES * 8] bytes, page aligned
; graphics_page  = high byte of pointer to first page of graphics

NUM_TILES  = 32                         ; number of tiles (1-32)
LCD_WIDTH  = 12                         ; width of screen (12 or 16)
MAP_WIDTH  = 16                         ; width of map
MAP_HEIGHT = 16                         ; height (power of 2 for best result)

#DEFINE TILE_MAP LD A,(BC)\ OUT ($11),A\ LD A,(DE)\ OR (HL)\ LD (BC),A\ INC C\ INC DE\ INC HL
#DEFINE TMAP_FINAL LD A,(BC)\ OUT ($11),A\ LD A,(DE)\ OR (HL)\ LD (BC),A\ INC C
#DEFINE TMAP_SPC(y) LD E,(IX+y)\ LD A,(BC)\ OUT ($11),A\ LD A,(IX+MAP_HEIGHT+y)\ LD L,A\ LD A,(DE)\ OR (HL)\ LD (BC),A\ INC C\ INC E\ INC L

render_map:

;=- graphic copy section -=

;- init -

   LD    A, (sox)                      ; BC = offset
   CPL                                 ;
   AND   $07                           ;
   LD    B, 0                          ;
   LD    C, A                          ;

   LD    HL, gfx_mask                  ; A = gfx mask
   ADD   HL, BC                        ;
   LD    A, (HL)                       ;
   LD    (gfx_modify1), A              ;
   CPL                                 ;
   LD    (gfx_modify2), A              ;

   LD    HL, snip_copy                 ; HL -> location of snip to copy
   ADD   HL, BC                        ;

   LD    DE, snip_dest                 ; DE -> destination of snip
   LD    C, 15                         ; BC = 15
   LDIR                                ;

;- copy -

   LD    HL, graphic_set               ; HL -> source
   LD    DE, graphics                  ; DE -> dest
   LD    B, NUM_TILES * 4              ; B = number of tiles

_mg_loop:
   LD    A, (HL)                       ; A = graphic
   INC   HL                            ; HL -> next byte of source

snip_dest:
   RLCA                                ; This part relocates
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   LD    C, A                          ; save data in C
   AND   %00001111                     ; mask
   LD    (DE), A                       ; (DE) = first part of data
   INC   D                             ; DE -> right half of byte
   XOR   C                             ; find other side of tile
   LD    (DE), A                       ; (DE) = second part of data
   LD    A, (HL)                       ; A = graphic
   RRCA                                ;
   RRCA                                ;
   RRCA                                ;

   RRCA                                ;

   INC   E                             ; DE -> next byte of dest
   INC   HL                            ; HL -> next byte of source

   LD    C, A                          ; ditto above
gfx_modify2 = $+1                      ;
   AND   %11110000                     ;
   LD    (DE), A                       ;
   DEC   D                             ;
   XOR   C                             ;
   LD    (DE), A                       ;
   INC   E                             ;

   DJNZ  _mg_loop                      ; loop

;=- map render section -= (46819 clocks)

;- init -

; *** must modify - change calculation to multiply ssx by MAP_HEIGHT ***
   LD    A, (ssx)                      ; A = ssy
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   RLCA                                ;
   LD    L, A                          ;
   AND   %00001111                     ;
   LD    H, A                          ;
   XOR   L                             ;
   LD    L, A                          ; HL = ssy * 16
   LD    A, (ssy)                      ;
   LD    C, A                          ; BC = ssx
   LD    A, (soy)                      ;
   ADD   A, -1                         ;
   ADC   HL, BC                        ;
   EX    DE, HL                        ; DE = ssy * 16 + ssx
; *** *** ***

;- render -

   LD    IX, mapbuffer - 1             ; HL -> mapbuffer + offset
   ADD   IX, DE                        ;

   LD    A, (soy)                      ; L = y offset
   AND   $07                           ;
   LD    C, A                          ;

   LD    HL, jr_offsets                ; 1st jump offset
   ADD   HL, BC                        ;
   LD    A, (HL)                       ;
   LD    (jump_ofs1), A                ;
   XOR   $3F                           ; 2nd jump offset
   LD    (jump_ofs2), A                ;

   EXX                                 ;
   LD    BC, graphmem                  ; BC' -> graphics memory
   LD    H, graphics_page              ; HL' -> graphics_page
   LD    D, graphics_page+1            ; DE' -> graphics_page+1
   EXX                                 ;

   LD    H, $20                        ; H = starting column
   LD    L, C                          ; L = starting row
   LD    BC, $0C10                     ; B = loops, C = lcd port
   LD    DE, MAP_HEIGHT                ; DE = height of map

;----------------------------------------------------------------------------
;[ draw_map ]                                              (3869 per column)
;----------------------------------------------------------------------------
; IX -> map
; L = starting row
; DE = height of map
; H = starting column
; L = starting row
;
; HL' -> graphics_page
; DE' -> graphics_page+1
; BC' -> graphmem

_draw_map_loop:
   OUT   (C), H                        ; write column number to port
   INC   H                             ; go to next column

                                       ; first portion: [77]     (74 to 459)
;- init the loop (74 clocks) ------------------------------------------------
   LD    A, L                          ; grab starting row
   EXX                                 ;
   LD    L, A                          ;
   ADD   A, (IX + 0)                   ;
   LD    E, A                          ; DE' -> graphic of tile on left
   LD    A, L                          ;
   ADD   A, (IX + MAP_HEIGHT + 0)      ;
   LD    L, A                          ; HL' -> graphic of tile on right
jump_ofs1 = $+1                        ;
   JR    jump_ofs1                     ; store 9 * soy here

   LD    A, (BC)                       ; tile_map (55 clocks)
   OUT   ($11), A                      ; out with the old (to the LCD :P)
   LD    A, (DE)                       ; in with the new
   OR    (HL)                          ; combine 2 chips
   LD    (BC), A                       ; save it
   INC   C                             ; BC' -> next byte of graphmem
   INC   DE                            ; DE' -> next row of graphic
   INC   HL                            ; HL' -> next row of graphic

   TILE_MAP                            ; ditto above
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;

   LD    A, (BC)                       ; special cased final (55 clocks)
   OUT   ($11), A                      ;   (37 clocks after LCD access)
   LD    A, (DE)                       ; in with the new
   OR    (HL)                          ; combine 2 chips
   LD    (BC), A                       ; save it
   INC   C                             ; BC' -> next byte of graphmem
   INC   BC                            ; delay
   DEC   BC                            ;
;----------------------------------------------------------------------------
                                       ; fixed portion: [539]     (3262)
                                       ; unrolled loop: [77/loop] (466/loop)
;----------------------------------------------------------------------------
   LD    E, (IX + 1)                   ; special cased tile_map (93 clocks)
   LD    A, (BC)                       ; out with the old
   OUT   ($11), A                      ;
   LD    A, (IX + MAP_HEIGHT + 1)      ; DE' -> graphic of tile on left
   LD    L, A                          ; HL' -> graphic of tile on right
   LD    A, (DE)                       ; in with the new
   OR    (HL)                          ; combine 2 chips
   LD    (BC), A                       ; save it
   INC   C                             ; BC' -> next byte of graphmem
   INC   E                             ; DE' -> next row of graphic
   INC   L                             ; HL' -> next row of graphic

   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map
;----------------------------------------------------------------------------
   TMAP_SPC(2)                         ; special cased tile_map
   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map
;----------------------------------------------------------------------------
   TMAP_SPC(3)                         ; special cased tile_map
   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map
;----------------------------------------------------------------------------
   TMAP_SPC(4)                         ; special cased tile_map
   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map
;----------------------------------------------------------------------------
   TMAP_SPC(5)                         ; special cased tile_map
   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map
;----------------------------------------------------------------------------
   TMAP_SPC(6)                         ; special cased tile_map
   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map
;----------------------------------------------------------------------------
   TMAP_SPC(7)                         ; special cased tile_map
   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TMAP_FINAL                          ; final tile_map

                                       ; final portion: [79]     (99 to 484)
;- init the loop (42 clocks) ------------------------------------------------
   LD    E, (IX + 8)                   ; DE' -> graphic of tile on left
   LD    A, (IX + MAP_HEIGHT + 8)      ;
   LD    L, A                          ; HL' -> graphic of tile on right
jump_ofs2 = $+1                        ;
   JR    jump_ofs2                     ; store 9 * (8 - soy) here

   TILE_MAP                            ; tile_map
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;
   TILE_MAP                            ;

   LD    A, (BC)                       ; special cased final (45 clocks)
   OUT   ($11), A                      ;   (27 clocks after LCD access)
   LD    A, (DE)                       ; in with the new
   OR    (HL)                          ; combine 2 chips
   LD    (BC), A                       ; save it
   INC   BC                            ; BC' -> next byte of graphmem
;----------------------------------------------------------------------------
   EXX                                 ;

   ADD   IX, DE                        ; IX -> next column in map

   DEC   B                             ; are we done looping?
   JP    NZ, _draw_map_loop            ;

   RET                                 ;

;- data -

gfx_mask:
   .DB   %01111111
   .DB   %00111111
   .DB   %00011111
   .DB   %00001111
   .DB   %00000111
   .DB   %00000011
   .DB   %00000001
   .DB   %00000000

jr_offsets:
   .DB  63, 0, 9,18,27,36,45,54

snip_copy:
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   RLCA
   LD    C, A
gfx_modify1 = $+1
   AND   %00001111
   LD    (DE), A
   INC   D
   XOR   C
   LD    (DE), A
   LD    A, (HL)
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
   RRCA
